/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.api;

import org.objectweb.jalisto.se.api.internal.SessionInternal;
import org.objectweb.jalisto.se.api.query.QueryManager;

import java.util.Collection;

/**
 * Session is the principale Jalisto interface for users.
 * It allows to define class, store, read, modify or remove obects.
 */
public interface Session {

    /**
     * Allows users to define structure of a persistent class.
     * 
     * @param classDescription The meta-description of the persistent class to define.
     */
    void defineClass(ClassDescription classDescription);

    /**
     * Allows users to remove the structure of a persistent class.
     * 
     * @param fullClassName The full class name of the persistent class to remove.
     */
    void removeClass(String fullClassName);

    /**
     * Check if a persistent class is define in Jalisto.
     * 
     * @param fullQualifiedClassName The full class name of the class to check.
     * @return true if there already is a class with this name define in the base.
     */
    boolean isClassDefined(String fullQualifiedClassName);

    /**
     * Get class name of a persistent instance. The user has to specify the identity of the persistent instance.
     * 
     * @param oid The identity of the persistent instance.
     * @return String : The full class name of the corresponding persistent class; null otherwise.
     */
    String getClassNameFor(Object oid);


    /**
     * Create a new identity for an instance of the given class.
     * 
     * @param objectClass The class of the expecting new persistent instance.
     * @return Object : The new identity of the expecting new persistent instance.
     */
    Object makeNewFileOid(Class objectClass);

    /**
     * Create a new identity for an instance of the given class.
     * 
     * @param objectClassName The full qualified class name of the expecting new persistent instance.
     * @return Object : The new identity of the expecting new persistent instance.
     */
    Object makeNewFileOid(String objectClassName);

    /**
     * Make a new instance of a given class persistent.
     * Also create a new identity for this instance and return it.
     * 
     * @param objectToCreate The array representation of the persistent object, according to the class meta-description.
     * @param objectClass    The class of the new persistent instance.
     * @return Object : The new identity of the expecting new persistent instance.
     */
    Object createObject(Object[] objectToCreate, Class objectClass);

    /**
     * Make a new instance of a given class persistent.
     * Also create a new identity for this instance and return it.
     * 
     * @param objectToCreate  The array representation of the persistent object, according to the class meta-description.
     * @param objectClassName The full qualified class name of the new persistent instance.
     * @return Object : The new identity of the expecting new persistent instance.
     */
    Object createObject(Object[] objectToCreate, String objectClassName);

    /**
     * Make a new instance of a given persistent class, with a given identity.
     * The identity must has been created before with method Object createObject(Object[] objectToCreate, Class objectClass). The class specified at this time must be the same than the instance class.
     * 
     * @param oid            The identity of the new instance.
     * @param objectToCreate The array representation of the persistent object, according to the class meta-description.
     * @return Object : The identity of the expecting new persistent instance.
     */
    Object createObject(Object oid, Object[] objectToCreate);

    /**
     * Read the object of the given identity in Jalisto.
     * 
     * @param oid The identity of the persistent instance.
     * @return Object[] : The array representation of the persistent object, according to the class meta-description.
     */
    Object[] readObjectByOid(Object oid);

    Collection readObjectsByOids(Collection oids);

     Object[] refreshObjectByOid(Object oid);

    /**
     * Tell if the datastore contains a persistent object of the given identity in Jalisto.
     * 
     * @param oid The identity of the persistent instance.
     * @return boolean : true if the datastore contains a persistent object with the given identity
     */
    boolean contains(Object oid);

    /**
     * Update a persistent instance with a given identity.
     * 
     * @param oid            The identity of the persistent instance.
     * @param objectToUpdate The array representation of the persistent object, according to the class meta-description.
     * @return Object : The identity of the persistent instance.
     */
    Object updateObjectByOid(Object oid, Object[] objectToUpdate);

    /**
     * Remove a persistent object of the given identity from Jalisto.
     * 
     * @param oid The identity of the persistent instance.
     */
    void deleteObjectByOid(Object oid);


    /**
     * Get identities of all persistent instance of a given class.
     * 
     * @param fullClassName The full name of the class.
     * @return Collection : A collection of all the oid of the persistent instances of the class.
     */
    Extent getExtent(String fullClassName);

    /**
     * Get identities of all persistent instance of a given class.
     * 
     * @param theClass The class.
     * @return Collection : A collection of all the oid of the persistent instances of the class.
     */
    Extent getExtent(Class theClass);


    /**
     * Tell if the Jalisto base has been created during the initialisation of the Session instance.
     * 
     * @return boolean : true if the base has been created during the initialisation of Session instance; false otherwise.
     */
    boolean isNewBase();

    /**
     * Try to optimize space in base files.
     */
    void reorganize();

    /**
     * The getQueryManager method returns the QueryManager instance associated with the Session instance.
     * 
     * @return QueryManager : The QueryManager instance associated with the Session instance.
     */
    QueryManager getQueryManager();

    /**
     * The currentTransaction method returns the Transaction instance associated with the Session instance.
     * 
     * @return Transaction : The Transaction instance associated with the Session instance.
     */
    Transaction currentTransaction();

    /**
     * Get a MetaRepository instance to work on Jalisto schema of persistent classes.
     * 
     * @return MetaRepository : the meta repository associated with Session instance.
     */
    MetaRepository getMetaRepository();

    /**
     * Get an internal view of Session instance.
     * Users don't have to use this interface for commun functionalities.
     * 
     * @return SessionInternal : A internal view of the Session instance.
     */
    SessionInternal getInternalSession();

    /**
     * Open the session. User can now work on the session.
     */
    void openSession();

    /**
     * Close the session. No more work will be done on this session.
     */
    void closeSession();

    boolean isOpen();

    void eraseStorage();
}
