/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.storage.raf.log;

import org.objectweb.jalisto.se.api.internal.Constants;
import org.objectweb.jalisto.se.storage.raf.*;
import org.objectweb.jalisto.se.storage.raf.log.backup.LogObject;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

public class RecordsFileLog extends RecordsFile {

    // The underlying file.
    private RandomAccessFile logFile;

    private boolean withCleanLog;

    public RecordsFileLog(String dbPath, boolean withCleanLog)
            throws IOException, RecordsFileException {
        super(dbPath);
        File fb = new File(dbPath + Constants.LOG_POSTFIX);
        if (fb.exists()) {
            throw new RecordsFileException("Database log already exits: " + dbPath + Constants.LOG_POSTFIX);
        }
        logFile = new RandomAccessFile(fb, "rw");
        this.withCleanLog = withCleanLog;
    }

    public RecordsFileLog(String dbPath, String accessFlags, boolean withCleanLog)
            throws IOException, RecordsFileException {
        super(dbPath, accessFlags);
        File fb = new File(dbPath + Constants.LOG_POSTFIX);
        if (!fb.exists()) {
            throw new RecordsFileException("Database log already exits: " + dbPath + Constants.LOG_POSTFIX);
        }
        logFile = new RandomAccessFile(fb, "rw");
        this.withCleanLog = withCleanLog;
    }

    public synchronized void close() throws IOException, RecordsFileException {
        try {
            super.close();
            logFile.close();
        } finally {
            logFile = null;
        }
    }

    /**
     * *******************************************  LOGGING  ************************************************
     */

    protected void writeSetFileLength(long newSize) throws IOException {
        if (file.length() < newSize) {
            LogObject.writeLogFileLengthModification(logFile, newSize - file.length());
        } else if (file.length() > newSize) {
            int suppressionSize = new Long(file.length() - newSize).intValue();
            byte[] datas = new byte[suppressionSize];
            file.seek(newSize);
            file.readFully(datas);
            LogObject.writeLogFileLengthModification(logFile, datas);
        }
    }

    protected void writeDatasInLog(long position, int dataLength) throws IOException {
        byte[] oldDatas = new byte[dataLength];
        file.seek(position);
        file.readFully(oldDatas);
        LogObject.writeLogRecord(logFile, position, oldDatas, dataLength);
    }

    protected void writeByteArray(long position, byte[] datas) throws IOException {
        writeDatasInLog(position, datas.length);
        super.writeByteArray(position, datas);
    }

    protected void writeOutputStream(long position, DbByteArrayOutputStream outputStream) throws IOException {
        writeDatasInLog(position, outputStream.size());
        super.writeOutputStream(position, outputStream);
    }

    protected void writeRecordWriter(RecordHeader header, RecordWriter recordWriter) throws IOException {
        long position = header.getDataPointer();
        int dataSize = header.getDataCapacity();
        if (position + dataSize >= file.length()) {
            int oldDataSize = (new Long(file.length() - 1 - position)).intValue();
            byte[] oldDatas = new byte[oldDataSize];
            file.seek(position);
            file.readFully(oldDatas);
            LogObject.writeLogRecord(logFile, position, oldDatas, recordWriter.getOut().size());
        } else {
            writeDatasInLog(position, recordWriter.getOut().size());
        }
        super.writeRecordWriter(header, recordWriter);
    }

    protected void writeRecordHeader(long position, RecordHeader recordHeader) throws IOException {
        DbByteArrayOutputStream byteOutputStream = DbByteArrayOutputStream.getInstance();
        recordHeader.write(byteOutputStream.getOutput());
        writeDatasInLog(position, byteOutputStream.size());
        byteOutputStream.reset();
        super.writeRecordHeader(position, recordHeader);
    }

    protected void writeInt(long position, int i) throws IOException {
        DbByteArrayOutputStream byteOutputStream = DbByteArrayOutputStream.getInstance();
        byteOutputStream.getOutput().writeInt(i);
        writeDatasInLog(position, byteOutputStream.size());
        byteOutputStream.reset();
        super.writeInt(position, i);
    }

    protected void writeLong(long position, long l) throws IOException {
        DbByteArrayOutputStream byteOutputStream = DbByteArrayOutputStream.getInstance();
        byteOutputStream.getOutput().writeLong(l);
        writeDatasInLog(position, byteOutputStream.size());
        byteOutputStream.reset();
        super.writeLong(position, l);
    }

    protected void writeUTF(long position, String s) throws IOException {
        DbByteArrayOutputStream byteOutputStream = DbByteArrayOutputStream.getInstance();
        byteOutputStream.getOutput().writeUTF(s);
        writeDatasInLog(position, byteOutputStream.size());
        byteOutputStream.reset();
        super.writeUTF(position, s);
    }

    public void startBackUp() throws IOException {
        if (withCleanLog) {
            logFile.seek(0);
            logFile.setLength(0);
        } else {
            logFile.seek(logFile.length());
        }
        LogObject.writeLogBoundary(logFile, true);
    }

    public void finishBackUp() throws IOException {
        LogObject.writeLogBoundary(logFile, false);
        if (withCleanLog) {
            logFile.setLength(logFile.getFilePointer());
        }
    }
}
