/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.tool;

import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.api.*;
import org.objectweb.jalisto.se.api.internal.InternalPhysicalFileAccess;
import org.objectweb.jalisto.se.api.internal.LogicalSystemPageAccess;
import org.objectweb.jalisto.se.api.internal.SessionInternal;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.ExtentImpl;
import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.impl.server.SessionImpl;
import org.objectweb.jalisto.se.impl.meta.InternalMetaRepositoryImpl;

import java.io.PrintWriter;
import java.util.Collection;
import java.util.Iterator;
import java.util.StringTokenizer;

/**
 * Utility class to look into a Jalisto base, specified by his properties file.
 * Call main method with no argument to print usage.
 */
public class JalistoViewer {

    public static void main(String[] args) {
        try {
            parseArgs(args);
            JalistoViewer viewer = new JalistoViewer(JalistoFactory.getInternalFactory().getSession(propFilePath),
                                             args,
                                             new PrintWriter(System.out));
            viewer.readBase();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private static void printUsage(PrintWriter writer, String message) {
        if ((message != null) && (!message.equals(""))) {
            writer.println(message);
        }
        writer.println(
                "java org.objectweb.jalisto.JalistoViewer <option> : print persistent instances from specified datastore");
        writer.println("\t" + FILE_OPTION + " <path> : specify the properties file of the base to inspect (mandatory)");
        writer.println("\t" + NO_INSTANCE_OPTION + " : doesn't show instance");
        writer.println("\t" + INSTANCE_OPTION + " <coma-separated floid list> : show only the floid-related instances");
        writer.println("\t" + META_OPTION + " : show the class meta-model");
        writer.println("\t" + STATE_OPTION + " : show the initial state of the base");
    }


    public JalistoViewer(Session session, String[] args, PrintWriter out) {
        try {
            parseArgs(args);
        } catch (Exception e) {
            printUsage(out, e.getMessage());
            out.flush();
            System.exit(0);
        }

        this.writer = out;
        if (session == null) {
            this.session = JalistoFactory.getSession(propFilePath);
        } else {
            this.session = session;
        }
        session.openSession();
        fileAccess = ((SessionInternal)session).getFileAccess();
        repository = (InternalMetaRepositoryImpl) session.getMetaRepository();
    }

    public void readBase() {
        if (withState) {
            printState(0);
        }
        session.currentTransaction().begin();
        Iterator classNameIterator = getAllClassNames().iterator();
        while (classNameIterator.hasNext()) {
            String className = (String) classNameIterator.next();
            Collection oids = getExtent(className);
            if (withMeta) {
                printClassMeta(1, className, oids);
            }
            if ((!withSelectedInstance) && (!noInstance)) {
                printClassInstances(1, className, oids);
            }
        }
        if (withSelectedInstance && (!noInstance)) {
            StringTokenizer tokenizer = new StringTokenizer(instanceList, ",");
            while (tokenizer.hasMoreTokens()) {
                long l = Long.parseLong(tokenizer.nextToken());
                LogicalOid floid = new LogicalOid(l);
                Short clid = new Short(floid.getClid());
                ClassDescription meta = repository.getPersistentClassMetaDescription(clid);
                printInstance(1, meta, floid);
            }
        }
        session.currentTransaction().commit();
        writer.flush();
    }

    private Collection getAllClassNames() {
        return repository.getAllClassNames();
    }

    private Collection getExtent(String fullClassName) {
        Extent extent = new ExtentImpl(repository.getClidFromClassName(fullClassName),
                                             (SessionInternal) session);
        return extent.readFully().collection();
    }

    private static String getTab(int nbrTab) {
        String tab = "";
        for (int i = 0; i < nbrTab; i++) {
            tab = tab + "\t";
        }
        return tab;
    }

    private static void parseArgs(String[] args) {
        if (args.length == 0) {
            throw new JalistoException("FileViewer : no arguments");
        }

        boolean hasFileOption = false;

        for (int i = 0; i < args.length; i++) {
            String o = args[i];

            if (o.equalsIgnoreCase(FILE_OPTION)) {
                propFilePath = args[i + 1];
                hasFileOption = true;
                i++;
            } else if (o.equalsIgnoreCase(INSTANCE_OPTION)) {
                instanceList = args[i + 1];
                i++;
                withSelectedInstance = true;
            } else if (o.equalsIgnoreCase(NO_INSTANCE_OPTION)) {
                noInstance = true;
            } else if (o.equalsIgnoreCase(META_OPTION)) {
                withMeta = true;
            } else if (o.equalsIgnoreCase(STATE_OPTION)) {
                withState = true;
            } else if (o.equalsIgnoreCase(HELP_OPTION)) {
                throw new JalistoException("FileViewer : print help option");
            } else {
                throw new JalistoException("FileViewer : invalid argument format : " + o);
            }
        }

        if (!hasFileOption) {
            throw new JalistoException("FileViewer : no -file option");
        }
    }

    private void printClassInstances(int nbrTab, String className, Collection oids) {
        Iterator oidsIt = oids.iterator();
        Object clid = repository.getClidFromClassName(className);
        ClassDescription meta = repository.getPersistentClassMetaDescription(clid);
        while (oidsIt.hasNext()) {
            LogicalOid floid = (LogicalOid) oidsIt.next();
            printInstance(nbrTab, meta, floid);
        }
        writer.println();
    }

    private void printClassMeta(int nbrTab, String className, Collection oids) {
        String tab = getTab(nbrTab);
        String tabPlus1 = getTab(nbrTab + 1);
        Object clid = repository.getClidFromClassName(className);
        ClassDescription meta = repository.getPersistentClassMetaDescription(clid);
        String nbrInstanceMessage;
        int size = oids.size();
        if (size == 0) {
            nbrInstanceMessage = "no instances";
        } else if (size == 1) {
            nbrInstanceMessage = "1 instance";
        } else {
            nbrInstanceMessage = String.valueOf(size) + " instances";
        }
        writer.println(className + " : " + nbrInstanceMessage);
        writer.println(tab + "Meta-description of class " + className);
        writer.println(tabPlus1 + "Instance number : " + nbrInstanceMessage);
        String[] fieldNames = meta.getFieldNames();
        for (int i = 0; i < fieldNames.length; i++) {
            writer.println(tabPlus1 + "field " + i + " name : " + fieldNames[i] +
                           ", type : " + meta.getFieldDescription(i).getType());
        }
        writer.println();
    }

    private void printInstance(int nbrTab, ClassDescription meta, LogicalOid floid) {
        String tab = getTab(nbrTab);
        String tabPlus1 = getTab(nbrTab + 1);
        if (meta == null) {
            writer.println(tab + "Instance number : " + floid);
            writer.println(tabPlus1 + "object null in base");
            writer.println();
            return;
        }
        writer.println(tab + "Instance of " + meta.getClassName() + " number : " + floid);
        String[] fields = meta.getFieldNames();
        Object[] o = session.readObjectByOid(floid);
        if (o != null) {
            for (int i = 0; i < o.length; i++) {
                String objectInString;
                if (o[i] instanceof String) {
                    objectInString = "\"" + o[i] + "\"";
                } else {
                    objectInString = String.valueOf(o[i]);
                }
                writer.println(tabPlus1 + fields[i] + " : " + objectInString);
            }
        } else {
            writer.println(tabPlus1 + "object null in base");
        }
        writer.println();
    }

    private void printState(int nbrTab) {
        String tab = getTab(nbrTab);
        String tabPlus1 = getTab(nbrTab + 1);
        InternalPhysicalFileAccess physical = fileAccess.getPhysicalAccess();
        JalistoProperties state = (JalistoProperties) physical.readFileObjectAt(SessionImpl.BSTATE_IFA).getData();
        Iterator propIt = state.getKeys().iterator();
        writer.println(tab + "State of base :");
        while (propIt.hasNext()) {
            String key = (String) propIt.next();
            writer.println(tabPlus1 + key + " = " + state.getProperty(key));
        }
        writer.println();
    }

    private Session session;
    private InternalMetaRepositoryImpl repository;
    private LogicalSystemPageAccess fileAccess;
    private PrintWriter writer;


    private static String propFilePath;
    private static String instanceList;
    private static boolean withMeta = false;
    private static boolean withState = false;
    private static boolean withSelectedInstance = false;
    private static boolean noInstance = false;
    private static final String FILE_OPTION = "-file";
    private static final String INSTANCE_OPTION = "-i";
    private static final String NO_INSTANCE_OPTION = "-ni";
    private static final String META_OPTION = "-m";
    private static final String STATE_OPTION = "-s";
    private static final String HELP_OPTION = "-h";

}
